# bot.py
from telegram.ext import Application, CommandHandler, MessageHandler, CallbackQueryHandler
from telegram.ext import filters
from telegram.error import TelegramError, NetworkError
from handlers import *
import config
from database import Session, ScheduledMessage
from datetime import datetime
import utils
import asyncio
import nest_asyncio
import time

# Apply nest_asyncio to handle environments with existing event loops (e.g., Jupyter)
nest_asyncio.apply()

# Proxy settings (uncomment and configure if needed)
# PROXY_URL = "http://your-proxy-url:port"  # Replace with your proxy URL
# If you're using a proxy, uncomment the line above and set your proxy URL

async def run_scheduler(application):
    while True:
        session = Session()
        for msg in session.query(ScheduledMessage).all():
            if datetime.now().strftime("%H:%M") == msg.time:
                await application.bot.send_message(msg.group_id, msg.text)
        birthdays = utils.check_birthday()
        for bday in birthdays:
            await application.bot.send_message(bday.group_id, f"🎉 {bday.user_id} को जन्मदिन मुबारक!")
        session.close()
        await asyncio.sleep(60)

async def error_handler(update: Update, context: CallbackContext):
    """Handle uncaught errors and notify OWNER_ID."""
    error = context.error
    error_message = f"बॉट में त्रुटि हुई:\n{str(error)}"
    print(f"Error occurred: {error_message}")  # Debug log
    try:
        await context.bot.send_message(
            chat_id=config.OWNER_ID,
            text=error_message
        )
    except Exception as e:
        print(f"Error sending error message to OWNER_ID: {str(e)}")
        print(f"Original error: {str(error)}")

async def main():
    # Initialize Application with proxy (uncomment proxy_url if using a proxy)
    application = Application.builder().token(config.BOT_TOKEN).build()
    # If using a proxy, uncomment the line below and set PROXY_URL
    # application = Application.builder().token(config.BOT_TOKEN).proxy_url(PROXY_URL).build()

    # Infinite retry logic for initialization
    retry_delay = 15  # Increased delay to 15 seconds
    attempt = 1
    while True:
        try:
            await application.initialize()
            print("बॉट सफलतापूर्वक शुरू हो गया!")
            break  # If successful, break out of retry loop
        except NetworkError as e:
            print(f"नेटवर्क त्रुटि: {str(e)}\n{retry_delay} सेकंड बाद पुनः कोशिश कर रहा हूँ... (प्रयास {attempt})")
            await asyncio.sleep(retry_delay)
            attempt += 1

    # Command Handlers
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("ban", ban, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("mute", mute, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("unmute", unmute, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("warn", warn, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("unwarn", unwarn, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("kick", kick, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("info", info, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("setrules", setrules, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("rules", rules, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("anon", anon, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("setlang", setlang))
    application.add_handler(CommandHandler("schedule", schedule_message, filters=filters.ChatType.GROUPS))
    application.add_handler(CommandHandler("clean", clean_old_messages, filters=filters.ChatType.GROUPS))

    # Message Handlers
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, welcome))
    application.add_handler(MessageHandler(filters.StatusUpdate.LEFT_CHAT_MEMBER, leave))
    application.add_handler(MessageHandler(filters.Text() & filters.ChatType.GROUPS, check_spam))
    application.add_handler(MessageHandler(filters.ALL & filters.ChatType.GROUPS, anti_flood))
    application.add_handler(MessageHandler(filters.UpdateType.EDITED_MESSAGE, log_edit))
    application.add_handler(MessageHandler(filters.VOICE, voice_command))
    application.add_handler(MessageHandler(filters.Text(), check_mood))
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, check_admin_status))
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, collect_group_members))
    application.add_handler(MessageHandler(filters.CONTACT, handle_contact))

    # Callback Query Handlers
    application.add_handler(CallbackQueryHandler(admin_panel, pattern="ban|mute|setrules"))

    # Error Handler
    application.add_error_handler(error_handler)

    # Start scheduler in background
    application.job_queue.run_repeating(
        lambda context: run_scheduler(application),
        interval=60,
        first=0
    )

    # Run the bot
    try:
        await application.run_polling()
    finally:
        # Ensure proper shutdown
        await application.shutdown()

def run_bot():
    # Get or create an event loop
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

    try:
        loop.run_until_complete(main())
    finally:
        loop.close()

if __name__ == '__main__':
    run_bot()
